package jp.gr.java_conf.coskx.ocv_gl2d;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView;
import android.opengl.GLUtils;
import android.util.Log;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.util.HashMap;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;


/**
 * Created by tommy on 2015/06/18.
 */
public class GLRenderer implements GLSurfaceView.Renderer {
    private static final String TAG =  "GLRenderer";

    private static final int texturemode = 0;
    //OpenGLのtextureを作成するためのイメージは正方形でその一辺の長さは２のべき乗である必要があった
    //ちょっと古いデバイスではこの縛りがあるため，それに対応しておく
    //texturemode = 1 texture作成のためのBitmapのサイズを512x512の固定値を使う
    //texturemode = 0 サイズの縛りなし

    //システム
    private final Context mContext;
    private float aspect;//アスペクト比

    private boolean texture_is_Ready = false;

    GLRenderer(final Context context) {
        mContext = context;
        Log.i(TAG,"GLRenderer()");
    }

    private int glprogram;
    private HashMap<String, Integer> glvars;

    private FloatBuffer vertex_buffer;
    private FloatBuffer texture_buffer;
    private float[][] texture_uvs;

    //サーフェイス生成時に呼ばれる
    @Override
    public void onSurfaceCreated(GL10 gl10,EGLConfig eglConfig) {
        Log.i(TAG,"onSurfaceCreated() start");
        //プログラムの生成
        String vs_code = "attribute vec2 A_position;  attribute vec2 A_texture_uv;  varying vec2 V_texture_uv;"
                + "void main(void) {gl_Position = vec4(A_position, 0.0, 1.0);  V_texture_uv = A_texture_uv;}";
        String fs_code = "precision mediump float;  uniform sampler2D texture0;  varying vec2 V_texture_uv;"
                + "void main() {gl_FragColor = texture2D(texture0, V_texture_uv);}";

        int v_shader = GLES20.glCreateShader(GLES20.GL_VERTEX_SHADER);
        int f_shader = GLES20.glCreateShader(GLES20.GL_FRAGMENT_SHADER);

        // Vertexシェーダーのコードをコンパイル
        GLES20.glShaderSource(v_shader, vs_code);
        GLES20.glCompileShader(v_shader);

        // Fragmentシェーダーのコードをコンパイル
        GLES20.glShaderSource(f_shader, fs_code);
        GLES20.glCompileShader(f_shader);

        // Programを作成
        glprogram = GLES20.glCreateProgram();

        // Programのシェーダーを設定
        GLES20.glAttachShader(glprogram, v_shader);
        GLES20.glAttachShader(glprogram, f_shader);

        GLES20.glLinkProgram(glprogram);
        GLES20.glUseProgram(glprogram);

        glvars = new  HashMap<String, Integer>();

        // gl変数を保存
        glvars.put("A_position", GLES20.glGetAttribLocation(glprogram, "A_position"));
        glvars.put("A_texture_uv", GLES20.glGetAttribLocation(glprogram, "A_texture_uv"));
        glvars.put("texture0", GLES20.glGetUniformLocation(glprogram, "texture0"));

        // テクスチャ0を有効化
        GLES20.glActiveTexture(GLES20.GL_TEXTURE0);

        int[] textures = new int[1];

        // テクスチャ作成しidをtextures[0]に保存
        GLES20.glGenTextures(1, textures, 0);

        // テクスチャ0にtextures[0]をバインド
        GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, textures[0]);

        // 縮小時の補間設定
        GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MIN_FILTER, GLES20.GL_LINEAR);
        // 拡大時の補間設定
        GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MAG_FILTER, GLES20.GL_LINEAR);

        // gl変数texture0に0を設定
        GLES20.glUniform1i(glvars.get("texture0"), 0);

        // 「全体」を覆うテクスチャ座標配列を作成
        // (0, 1)-(1, 0)の長方形頂点データ作成 ただしX軸は右が正，Y軸は下向きが正
        //左下，右下，左上，右上の順 (0, 1) - (1, 1) - (0, 0) - (1, 0) rotationindex:0 のとき
        //左下，右下，左上，右上の順 (1, 1) - (1, 0) - (0, 1) - (0, 0) rotationindex:1 のとき
        //左下，右下，左上，右上の順 (1, 0) - (0, 0) - (1, 1) - (0, 1) rotationindex:2 のとき
        //左下，右下，左上，右上の順 (0, 0) - (0, 1) - (1, 0) - (1, 1) rotationindex:3 のとき
        //rotationindex 0:なにもしない　1:CW90回す　2:180回す　3:CCW90回す
        texture_uvs = new float[][]{{0f, 1f, 1f, 1f, 0f, 0f, 1f, 0f},
                {1f, 1f, 1f, 0f, 0f, 1f, 0f, 0f},
                {1f, 0f, 0f, 0f, 1f, 1f, 0f, 1f},
                {0f, 0f, 0f, 1f, 1f, 0f, 1f, 1f}
        };
        Log.i(TAG,"onSurfaceCreated() texture_uvs.length = " + texture_uvs.length);

        ByteBuffer tex_bb = ByteBuffer.allocateDirect(texture_uvs[0].length * 4);
        tex_bb.order(ByteOrder.nativeOrder());

        texture_buffer = tex_bb.asFloatBuffer();
        texture_buffer.put(texture_uvs[0]);
        texture_buffer.position(0);

        // (1, 1)-(-1, -1)の長方形頂点データ作成 ただしX軸は右が正，Y軸は上向きが正
        //左下，右下，左上，右上の順 (-1, -1) - (1, -1) - (-1, 1) - (1, 1)
        float[] vertex_array = {-1f, -1f, 1f, -1f, -1f, 1f, 1f, 1f};

        // 頂点データをバッファに格納
        ByteBuffer vt_bb = ByteBuffer.allocateDirect(vertex_array.length * 4);
        vt_bb.order(ByteOrder.nativeOrder());

        vertex_buffer = vt_bb.asFloatBuffer();
        vertex_buffer.put(vertex_array);
        vertex_buffer.position(0);


        //背景色の設定
        GLES20.glClearColor(0f, 0f, 0.2f, 1.0f);

        // 背景とのブレンド方法を設定します。
        GLES20.glEnable(GLES20.GL_BLEND);
        GLES20.glBlendFunc(GLES20.GL_SRC_ALPHA, GLES20.GL_ONE_MINUS_SRC_ALPHA);    // 単純なアルファブレンド

        // 長方形のテクスチャ変数設定
        GLES20.glEnableVertexAttribArray(glvars.get("A_texture_uv"));
        GLES20.glVertexAttribPointer(glvars.get("A_texture_uv"), 2, GLES20.GL_FLOAT, false, 0, texture_buffer);

        // 長方形の頂点変数設定
        GLES20.glEnableVertexAttribArray(glvars.get("A_position"));
        GLES20.glVertexAttribPointer(glvars.get("A_position"), 2, GLES20.GL_FLOAT, false, 0, vertex_buffer);

        CVbitmap = null;
        bitmapbusy = 0;
        Log.i(TAG,"onSurfaceCreated() completed");
        // テクスチャ用Bitmap作成
        bmptest = Bitmap.createBitmap(32, 32, Bitmap.Config.ARGB_8888);

        // テクスチャ用の画像を描画
        for (int y = 0;y < 32;y++) {
            for (int x = 0;x < 32;x++) {
                bmptest.setPixel(x, y, Color.argb(255, 0, 0, 128));
            }
        }

    }

    private Bitmap bmptest;

    //カメラの向きが確定してから呼び出され，再設定される
    //mScaleW, mScaleH はデバイス固定の縦横倍率
    public void makeSettings(int rotindex, float mScaleW, float mScaleH) {
        Log.i(TAG,"makeSettings()");
        texture_buffer.put(texture_uvs[rotindex]);
        texture_buffer.position(0);
        // 長方形のテクスチャ変数設定
        GLES20.glEnableVertexAttribArray(glvars.get("A_texture_uv"));
        GLES20.glVertexAttribPointer(glvars.get("A_texture_uv"), 2, GLES20.GL_FLOAT, false, 0, texture_buffer);

        float w = 1f, h = 1f;
        if (rotindex%2 == 0) {//横位置
            if (mScaleW<mScaleH) h = mScaleW/mScaleH;
            else w = mScaleH/mScaleW;
        } else {//縦位置
            if (mScaleW<mScaleH) w = mScaleW/mScaleH;
            else h = mScaleH/mScaleW;
        }
        float[] vertex_array = {-w, -h, w, -h, -w, h, w, h};
        vertex_buffer.put(vertex_array);
        vertex_buffer.position(0);
        // 長方形の頂点変数設定
        GLES20.glEnableVertexAttribArray(glvars.get("A_position"));
        GLES20.glVertexAttribPointer(glvars.get("A_position"), 2, GLES20.GL_FLOAT, false, 0, vertex_buffer);

    }

    //画面サイズ変更時に呼ばれる
    @Override
    public void onSurfaceChanged(GL10 gl10,int w,int h) {
        Log.i(TAG,"onSurfaceChanged()");
        //ビューポート変換
        GLES20.glViewport(0,0,w,h);
        aspect=(float)w/(float)h;
        CVbitmap = null;
        bitmapbusy = 0;
    }

    //毎フレーム描画時に呼ばれる
    @Override
    public void onDrawFrame(GL10 glUnused) {
        //if (!validProgram) return;
        //画面のクリア
        GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT |
                GLES20.GL_DEPTH_BUFFER_BIT);
        Log.i(TAG,"onDrawFrame()");
        if (bitmapbusy == 0) {
            // カラーバッファクリア
            GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT);
            GLUtils.texImage2D(GLES20.GL_TEXTURE_2D, 0, bmptest, 0);
            // 長方形を描画
            GLES20.glDrawArrays(GLES20.GL_TRIANGLE_STRIP, 0, 4);
            return;
        }

        if (CVbitmap!=null && texture_is_Ready) {
            // カラーバッファクリア
            GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT);
            GLUtils.texImage2D(GLES20.GL_TEXTURE_2D, 0, CVbitmap, 0);
            // 長方形を描画
            GLES20.glDrawArrays(GLES20.GL_TRIANGLE_STRIP, 0, 4);

            texture_is_Ready = false;
            bitmapbusy = 0;
        }
    }

    private Bitmap CVbitmap;
    private int bitmapbusy=0;
    public boolean requesttoshow(Bitmap bitmap) {
        Log.i(TAG,"requesttoshow()");
        if (bitmapbusy != 0) return false; //失敗
        bitmapbusy = 1;
        if (texturemode == 0) {
            CVbitmap = bitmap.copy(Bitmap.Config.ARGB_8888, true);
        Log.i(TAG,"requesttoshow bitmap w x h =" + bitmap.getWidth() +" x " + bitmap.getHeight());
        } else {
            //aspectHperW = (float)bitmap.getHeight()/(float) bitmap.getWidth(); //必ず1より小さい
            CVbitmap = Bitmap.createScaledBitmap(CVbitmap, 512, 512, true);
        }
        Log.i(TAG, "requesttoshow() CVbitmap = " + CVbitmap.toString());
        bitmapbusy = 2;
        texture_is_Ready = true;
        return true; //成功
    }

}

