package jp.gr.java_conf.coskx.ocv_gl2d;

import android.annotation.TargetApi;
import android.app.Activity;
import android.content.pm.PackageManager;
import android.media.ImageReader;
import android.opengl.GLSurfaceView;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.util.Size;
import android.view.SurfaceView;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;

import org.opencv.core.Mat;

import static android.Manifest.permission.CAMERA;

import java.util.List;

public class MainActivity extends AppCompatActivity implements CameraBridgeViewBaseGL.CvCameraViewListener2, View.OnClickListener {
//public class MainActivity extends Activity implements CameraBridgeViewBaseGL.CvCameraViewListener2, View.OnClickListener {
    private static final String TAG = "MainActivity";
    private static final int CAMERA_PERMISSION_REQUEST_CODE = 200;

    private JavaCamera2ViewGL mOpenCvCameraView;
    private MyGLSurfaceView glView;
    private boolean useGL = true;
    private Button toggleBtn;
    private Button resolutionBtn;
    private TextView info1, info2;

    @Override
    protected void onStart() {
        super.onStart();
        boolean havePermission = true;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (checkSelfPermission(CAMERA) != PackageManager.PERMISSION_GRANTED) {
                requestPermissions(new String[]{CAMERA}, CAMERA_PERMISSION_REQUEST_CODE);
                havePermission = false;
            }
        }
        if (havePermission) {
            mOpenCvCameraView.setCameraPermissionGranted();
        }
    }

    @Override
    @TargetApi(Build.VERSION_CODES.M)
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        if (requestCode == CAMERA_PERMISSION_REQUEST_CODE && grantResults.length > 0
                && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            mOpenCvCameraView.setCameraPermissionGranted();
        }
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        Log.i(TAG, "called onCreate");
        super.onCreate(savedInstanceState);
        System.loadLibrary("opencv_java4");
        setContentView(R.layout.activity_main);

        toggleBtn = (Button) findViewById(R.id.toggle);
        toggleBtn.setOnClickListener(this);
        resolutionBtn = (Button) findViewById(R.id.resolution);
        resolutionBtn.setOnClickListener(this);
        info1 = (TextView) findViewById(R.id.info1);
        info2 = (TextView) findViewById(R.id.info2);

        //GLサーフェスビュー
        glView=(MyGLSurfaceView)findViewById(R.id.id_myGLview);

        glView.setRenderMode(GLSurfaceView.RENDERMODE_WHEN_DIRTY);
        //これを有効にすると，描画させたいタイミングで、
        // 描画させたいタイミングで、GLSurfaceViewのrequestRender()を呼び出す必要がある。

        mOpenCvCameraView = (JavaCamera2ViewGL) findViewById(R.id.camera_view);
        mOpenCvCameraView.setVisibility(SurfaceView.VISIBLE); //これはINVISIBLEにしてはいけない
        mOpenCvCameraView.setCvCameraViewListener(this);
        mOpenCvCameraView.setGLRendererInstance(glView.renderer);
        mOpenCvCameraView.setMyGLSurfaceView(glView);
        mOpenCvCameraView.setActivity(MainActivity.this); //デバイスの向きを取得するのに使う
        useGL = true;

    }

    public void onClick(View v) {
        if (v == toggleBtn){
            useGL = !useGL;
            mOpenCvCameraView.setUseGL(useGL);

            if (useGL) {
                glView.setVisibility(View.VISIBLE);
                //mOpenCvCameraView.setVisibility(View.INVISIBLE);
                info1.setText(" GLSurfaceView ");
            } else {
                glView.setVisibility(View.INVISIBLE);
                //mOpenCvCameraView.setVisibility(View.VISIBLE);
                info1.setText(" JavaCamera2View ");
            }
        } else if (v == resolutionBtn) {
            resolutionBtn.setEnabled(false);
            setResolution();
        }
    }

    int selectednum = 0;

    private void setResolution() {

        java.util.List<android.util.Size> sizes_list = mOpenCvCameraView.getSizesList();
        int number = sizes_list.size();
        String[] menuitem = new String[number];
        android.util.Size[] sizes = new android.util.Size[number];
        for (int i=0; i<number; i++) {
            sizes[i] = sizes_list.get(i);
            menuitem[i] = "" + sizes[i].getWidth() + " x " + sizes[i].getHeight();
        }

        new android.app.AlertDialog.Builder(MainActivity.this)
                .setTitle("Select Bluetooth Device")
                .setCancelable(false)
                .setSingleChoiceItems(menuitem, selectednum, (dialog, item) -> {
                    //アイテムを選択したらここに入る
                    selectednum = item;
                })
                .setPositiveButton("Select", (dialog, id) -> {
                    //Selectを押したらここに入る
                    mOpenCvCameraView.disableView();
                    try {
                        Thread.sleep(100); //ミリ秒Sleepする
                    } catch (InterruptedException e) {
                    }
                    Log.i(TAG, "setResolution() Selected selectednum " + selectednum);
                    mOpenCvCameraView.setMaxFrameSize(sizes[selectednum].getWidth(),sizes[selectednum].getHeight());
                    mOpenCvCameraView.enableViewWithclear();
                    resolutionBtn.setEnabled(true);
                })
                .setNegativeButton("Cancel", (dialog, id) -> {
                    //Cancelを押したらここに入る
                    resolutionBtn.setEnabled(true);
                })
                .show();

    }

    //アクティビティレジューム時に呼ばれる
    @Override
    public void onResume() {
        super.onResume();
        glView.onResume();
        mOpenCvCameraView.enableView();

        if (useGL) {
            glView.setVisibility(View.VISIBLE);
            //mOpenCvCameraView.setVisibility(View.INVISIBLE);
            info1.setText(" GLSurfaceView ");
        } else {
            glView.setVisibility(View.INVISIBLE);
            //mOpenCvCameraView.setVisibility(View.VISIBLE);
            info1.setText(" JavaCamera2View ");
        }
        mOpenCvCameraView.setFPSTextView(info2);

    }

    //アクティビティポーズ時に呼ばれる
    @Override
    public void onPause() {
        super.onPause();
        glView.onPause();

        if (mOpenCvCameraView != null)
            mOpenCvCameraView.disableView();
    }

    public void onDestroy() {
        super.onDestroy();
        if (mOpenCvCameraView != null)
            mOpenCvCameraView.disableView();
    }

    public void onCameraViewStarted(int width, int height) {}

    public void onCameraViewStopped() {}

    public Mat onCameraFrame(CameraBridgeViewBaseGL.CvCameraViewFrame inputFrame) {
        return inputFrame.rgba();
    }

}
